"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GameData = void 0;
const constant_1 = require("@skeldjs/constant");
const protocol_1 = require("@skeldjs/protocol");
const Networkable_1 = require("../Networkable");
const PlayerInfo_1 = require("../misc/PlayerInfo");
const component_1 = require("./component");
const events_1 = require("../events");
const endgame_1 = require("../endgame");
/**
 * Represents a room object containing data about players.
 *
 * See {@link GameDataEvents} for events to listen to.
 */
class GameData extends Networkable_1.Networkable {
    constructor(room, spawnType, netid, ownerid, flags, data) {
        super(room, spawnType, netid, ownerid, flags, data);
        this.players || (this.players = new Map);
    }
    Awake() {
        for (const [, player] of this.room.players) {
            if (player.playerId)
                this.add(player.playerId);
        }
    }
    getComponent(component) {
        if (component === GameData) {
            return this.components[0];
        }
        if (component === component_1.VoteBanSystem) {
            return this.components[1];
        }
        return super.getComponent(component);
    }
    get owner() {
        return super.owner;
    }
    /**
     * Resolve some player identifier to a definite player info object.
     * @param resolvable The player identifier to resolve.
     * @returns A definite player info object.
     */
    resolvePlayerData(resolvable) {
        const resolved = this.room.resolvePlayerId(resolvable);
        if (resolved === undefined)
            return undefined;
        return this.players.get(resolved);
    }
    Deserialize(reader, spawn = false) {
        if (!this.players)
            this.players = new Map;
        if (spawn) {
            const num_players = reader.upacked();
            for (let i = 0; i < num_players; i++) {
                const playerId = reader.uint8();
                const player = PlayerInfo_1.PlayerInfo.Deserialize(reader, this, playerId);
                this.players.set(player.playerId, player);
            }
        }
        else {
            while (reader.left) {
                const [playerId, preader] = reader.message();
                const player = this.players.get(playerId);
                if (player) {
                    player.Deserialize(preader);
                }
                else {
                    const player = PlayerInfo_1.PlayerInfo.Deserialize(preader, this, playerId);
                    this.players.set(player.playerId, player);
                }
            }
        }
    }
    Serialize(writer, spawn = false) {
        let flag = false;
        if (spawn) {
            writer.upacked(this.players.size);
        }
        for (const [playerid, player] of this.players) {
            if ((1 << playerid) & this.dirtyBit || spawn) {
                if (spawn) {
                    writer.uint8(player.playerId);
                    writer.write(player);
                }
                else {
                    writer.begin(player.playerId);
                    writer.write(player);
                    writer.end();
                }
                flag = true;
            }
        }
        this.dirtyBit = 0;
        return flag;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.SetTasks:
                this._handleSetTasks(rpc);
                break;
        }
    }
    /**
     * Make the player data dirty and update on the next FixedUpdate.
     * @param resolvable The player to make dirty.
     */
    update(resolvable) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            this.dirtyBit |= 1 << player.playerId;
        }
    }
    /**
     * Change the name of a player (Will not update on clients, use {@link PlayerControl.setName}).
     * @param resolvable The player to change the name of.
     * @param name The name to change to.
     * @example
     *```typescript
     * room.gamedata.setName(player, "weakeyes");
     * ```
     */
    setName(resolvable, name) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            player.name = name;
            this.update(player);
        }
    }
    /**
     * Change the colour of a player (Will not update on clients, use {@link PlayerControl.setColor}).
     * @param resolvable The player to change the colour of.
     * @param color The colour to change to.
     * @example
     *```typescript
     * room.gamedata.setColor(player, ColorID.Blue);
     * ```
     */
    setColor(resolvable, color) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            player.color = color;
            this.update(player);
        }
    }
    /**
     * Change the hat of a player.
     * @param resolvable The player to change the hat of.
     * @param hat The hat to change to.
     * @example
     *```typescript
     * room.gamedata.setHat(player, HatID.TopHat);
     * ```
     */
    setHat(resolvable, hat) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            player.hat = hat;
            this.update(player);
        }
    }
    /**
     * Change the skin of a player.
     * @param resolvable The player to change the skin of.
     * @param skin The skin to change to.
     * @example
     *```typescript
     * room.gamedata.setSkin(player, SkinID.Mechanic);
     * ```
     */
    setSkin(resolvable, skin) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            player.skin = skin;
            this.update(player);
        }
    }
    /**
     * Change the pet of a player.
     * @param resolvable The player to change the pet of.
     * @param skin The pet to change to.
     * @example
     *```typescript
     * room.gamedata.setPet(player, PetID.MiniCrewmate);
     * ```
     */
    setPet(resolvable, pet) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            player.pet = pet;
            this.update(player);
        }
    }
    async _handleSetTasks(rpc) {
        const playerData = this.players.get(rpc.playerid);
        if (playerData) {
            const oldTasks = playerData.taskIds;
            this._setTasks(playerData, rpc.taskids);
            const playerTasks = playerData.taskIds;
            const ev = await this.emit(new events_1.GameDataSetTasksEvent(this.room, this, playerData, oldTasks, playerTasks));
            playerData.taskIds = ev.alteredTasks;
            if (ev.alteredTasks !== playerTasks) {
                this._rpcSetTasks(playerData, playerData.taskIds);
            }
        }
    }
    _setTasks(player, taskIds) {
        player.taskIds = taskIds;
        player.taskStates = taskIds.map((id, i) => new PlayerInfo_1.TaskState(i, false));
    }
    _rpcSetTasks(player, taskIds) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetTasksMessage(player.playerId, taskIds)));
    }
    /**
     * Set the tasks of a player.
     * @param player The player to set the tasks of.
     * @param taskIds The tasks to set.
     * @example
     *```typescript
     * room.gamedata.setTasks(player, [
     *   TheSkeldTask.ReactorUnlockManifolds,
     *   TheSkeldTask.ElectricDownloadData,
     *   TheSkeldTask.ShieldsPrimeShields,
     *   TheSkeldTask.NavigationDownloadData
     * ]);
     * ```
     */
    async setTasks(player, taskIds) {
        const playerData = this.resolvePlayerData(player);
        if (playerData) {
            const oldTasks = playerData.taskIds;
            this._setTasks(playerData, taskIds);
            const ev = await this.emit(new events_1.GameDataSetTasksEvent(this.room, this, playerData, oldTasks, playerData.taskIds));
            this._setTasks(playerData, ev.alteredTasks);
            if (playerData.taskIds !== oldTasks) {
                this._rpcSetTasks(playerData, playerData.taskIds);
                this.update(playerData);
            }
        }
    }
    /**
     * Mark a player's task as complete.
     * @param resolvable The player of the tasks to mark complete.
     * @param taskIdx The index of the player's tasks to mark complete.
     * @example
     *```typescript
     * // Complete all of a player's tasks.
     * for (let i = 0; i < player.info.tasks.length; i++) {
     *   room.gamedata.completeTask(player, i);
     * }
     * ```
     */
    completeTask(resolvable, taskIdx) {
        const player = this.resolvePlayerData(resolvable);
        if (player) {
            const task = player.taskStates[taskIdx];
            if (task) {
                task.completed = true;
            }
            let totalTasks = 0;
            let completeTasks = 0;
            const taskStates = new Map;
            for (const [, playerInfo] of this.players) {
                if (!playerInfo.isDisconnected && !playerInfo.isImpostor) {
                    const states = [];
                    taskStates.set(playerInfo.playerId, states);
                    for (const task of playerInfo.taskStates) {
                        totalTasks++;
                        if (task.completed) {
                            completeTasks++;
                        }
                        states.push({
                            taskId: playerInfo.taskIds[task.taskidx],
                            completed: task.completed
                        });
                    }
                }
            }
            if (totalTasks > 0 && completeTasks >= totalTasks) {
                this.room.registerEndGameIntent(new endgame_1.EndGameIntent(endgame_1.AmongUsEndGames.TasksComplete, constant_1.GameOverReason.HumansByTask, {
                    totalTasks,
                    completeTasks,
                    taskStates
                }));
            }
        }
    }
    /**
     * Add a player to player data.
     * @param playerId The player ID of the player to add.
     * @example
     *```typescript
     * // Get an available player ID and add it to the gamedata.
     * const playerId = room.getAvailablePlayerID();
     * room.gamedata.add(playerId);
     * ```
     */
    async add(playerId) {
        const playerInfo = PlayerInfo_1.PlayerInfo.createDefault(this, playerId);
        this.players.set(playerId, playerInfo);
        const ev = await this.emit(new events_1.GameDataAddPlayerEvent(this.room, this, playerInfo));
        if (ev.reverted) {
            this.players.delete(playerId);
        }
        else {
            this.update(playerId);
        }
        return playerInfo;
    }
    /**
     * Remove player data from the game data.
     * @param resolvable The player to remove.
     */
    async remove(resolvable) {
        const playerInfo = this.resolvePlayerData(resolvable);
        if (playerInfo) {
            const wasMarked = this.dirtyBit & (1 << playerInfo.playerId);
            if (wasMarked) {
                this.dirtyBit ^= 1 << playerInfo.playerId;
            }
            this.players.delete(playerInfo.playerId);
            const ev = await this.emit(new events_1.GameDataRemovePlayerEvent(this.room, this, playerInfo));
            if (ev.reverted) {
                this.players.set(playerInfo.playerId, playerInfo);
                this.update(playerInfo.playerId);
            }
        }
    }
}
exports.GameData = GameData;
//# sourceMappingURL=GameData.js.map