/// <reference types="node" />
import { HazelReader, HazelWriter } from "@skeldjs/util";
import { SpawnType } from "@skeldjs/constant";
import { BaseRpcMessage } from "@skeldjs/protocol";
import { ExtractEventTypes } from "@skeldjs/events";
import { Networkable, NetworkableEvents, NetworkableConstructor } from "../Networkable";
import { PlayerDataResolvable, Hostable } from "../Hostable";
import { PlayerVoteArea } from "../misc/PlayerVoteArea";
import { MeetingHudCloseEvent, MeetingHudVoteCastEvent, MeetingHudClearVoteEvent, MeetingHudVotingCompleteEvent } from "../events";
import { PlayerData } from "../PlayerData";
export interface MeetingHudData {
    voteStates: Map<number, PlayerVoteArea>;
    tie?: boolean;
    exilied?: PlayerData;
}
export declare type MeetingHudEvents<RoomType extends Hostable = Hostable> = NetworkableEvents<RoomType> & ExtractEventTypes<[
    MeetingHudVoteCastEvent<RoomType>,
    MeetingHudClearVoteEvent<RoomType>,
    MeetingHudVotingCompleteEvent<RoomType>,
    MeetingHudCloseEvent<RoomType>
]>;
export declare class MeetingHud<RoomType extends Hostable = Hostable> extends Networkable<MeetingHudData, MeetingHudEvents<RoomType>, RoomType> implements MeetingHudData {
    /**
     * The dirty vote states to be updated on the next fixed update.
     */
    dirtyBit: number;
    /**
     * The vote states in the meeting hud.
     */
    voteStates: Map<number, PlayerVoteArea<RoomType>>;
    /**
     * Whether the vote resulted in a tie.
     */
    tie?: boolean;
    /**
     * The player that was exiled, if any.
     */
    exiled?: PlayerData<RoomType>;
    ranOutOfTimeTimeout?: NodeJS.Timeout;
    constructor(room: RoomType, spawnType: SpawnType, netid: number, ownerid: number, flags: number, data?: HazelReader | MeetingHudData);
    Awake(): void;
    getComponent<T extends Networkable>(component: NetworkableConstructor<T>): T | undefined;
    get owner(): RoomType;
    Deserialize(reader: HazelReader, spawn?: boolean): void;
    Serialize(writer: HazelWriter, spawn?: boolean): boolean;
    HandleRpc(rpc: BaseRpcMessage): Promise<void>;
    private _handleClose;
    private _close;
    private _rpcClose;
    /**
     * Close the meeting hud for all clients. This is a host-only operation on
     * official servers.
     */
    close(): void;
    checkForVoteComplete(isTimeout: boolean): void;
    private _handleCastVote;
    private _castVote;
    private _rpcCastVote;
    /**
     * Cast a vote on behalf of a user (or yourself). Casting for another player
     * other than the player calling the function is a host-only operation on
     * official servers.
     * @param voter The player who is voting.
     * @param suspect The player to vote for.
     * @example
     *```typescript
     * // Make everyone vote a certain player.
     * for ([ clientid, player ] of room.players) {
     *   if (player !== suspect) {
     *     room.meetinghud.castVote(player, suspect);
     *   }
     * }
     * ```
     */
    castVote(voter: PlayerDataResolvable, suspect: PlayerDataResolvable | "skip"): Promise<void>;
    private _handleClearVote;
    private _clearVote;
    private _rpcClearVote;
    /**
     * Remove someone's vote (usually due to the player they voted for getting disconnected).
     * This is a host-only operation on official servers.
     * @param resolvable The player to remove the vote of.
     */
    clearVote(voter: PlayerDataResolvable): Promise<void>;
    private _handleVotingComplete;
    private _votingComplete;
    private _rpcVotingComplete;
    /**
     * End the meeting with specified results. This is a host-only operation
     * on official servers.
     * @param tie Whether this meeting resulted in a tie of votes.
     * @param exiled The player that was ejected, if any.
     */
    votingComplete(tie?: boolean, exiled?: PlayerDataResolvable): void;
}
