"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayerControl = void 0;
const protocol_1 = require("@skeldjs/protocol");
const constant_1 = require("@skeldjs/constant");
const events_1 = require("../events");
const Networkable_1 = require("../Networkable");
const AirshipStatus_1 = require("./AirshipStatus");
const LobbyBehaviour_1 = require("./LobbyBehaviour");
const systems_1 = require("../systems");
const component_1 = require("./component");
const endgame_1 = require("../endgame");
/**
 * Represents a player object for interacting with the game and other players.
 *
 * See {@link PlayerControlEvents} for events to listen to.
 */
class PlayerControl extends Networkable_1.Networkable {
    constructor(room, spawnType, netid, ownerid, flags, data) {
        var _a;
        super(room, spawnType, netid, ownerid, flags, data);
        this.lastStartCounter = 0;
        (_a = this.isNew) !== null && _a !== void 0 ? _a : (this.isNew = true);
        this.playerId || (this.playerId = 0);
        this.player = this.owner;
    }
    Awake() {
        this.playerId || (this.playerId = this.room.getAvailablePlayerID());
        if (this.room.gameData) {
            this.room.gameData.add(this.playerId);
        }
        if (this.isNew) {
            if (this.room.lobbyBehaviour) {
                const spawnPosition = LobbyBehaviour_1.LobbyBehaviour.spawnPositions[this.playerId % LobbyBehaviour_1.LobbyBehaviour.spawnPositions.length];
                const offsetted = spawnPosition
                    .add(spawnPosition.negate().normalize().div(4));
                this.getComponent(component_1.CustomNetworkTransform).snapTo(offsetted);
            }
            else if (this.room.shipStatus) {
                const spawnPosition = this.room.shipStatus.getSpawnPosition(this.playerId, true);
                this.getComponent(component_1.CustomNetworkTransform).snapTo(spawnPosition);
            }
        }
    }
    getComponent(component) {
        if (component === PlayerControl) {
            return this.components[0];
        }
        if (component === component_1.PlayerPhysics) {
            return this.components[1];
        }
        if (component === component_1.CustomNetworkTransform) {
            return this.components[2];
        }
        return undefined;
    }
    Deserialize(reader, spawn = false) {
        if (spawn) {
            this.isNew = reader.bool();
        }
        this.playerId = reader.uint8();
    }
    Serialize(writer, spawn = false) {
        if (spawn) {
            writer.bool(this.isNew);
            this.isNew = false;
        }
        writer.uint8(this.playerId);
        return true;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.CompleteTask:
                await this._handleCompleteTask(rpc);
                break;
            case constant_1.RpcMessageTag.SyncSettings:
                await this._handleSyncSettings(rpc);
                break;
            case constant_1.RpcMessageTag.SetInfected:
                await this._handleSetImpostors(rpc);
                break;
            case constant_1.RpcMessageTag.CheckName:
                if (this.room.hostIsMe) {
                    await this._handleCheckName(rpc);
                }
                break;
            case constant_1.RpcMessageTag.CheckColor:
                if (this.room.hostIsMe) {
                    await this._handleCheckColor(rpc);
                }
                break;
            case constant_1.RpcMessageTag.SetName:
                await this._handleSetName(rpc);
                break;
            case constant_1.RpcMessageTag.SetColor:
                await this._handleSetColor(rpc);
                break;
            case constant_1.RpcMessageTag.SetHat:
                await this._handleSetHat(rpc);
                break;
            case constant_1.RpcMessageTag.SetSkin:
                await this._handleSetSkin(rpc);
                break;
            case constant_1.RpcMessageTag.ReportDeadBody:
                if (this.room.hostIsMe) {
                    await this._handleReportDeadBody(rpc);
                }
                break;
            case constant_1.RpcMessageTag.MurderPlayer:
                await this._handleMurderPlayer(rpc);
                break;
            case constant_1.RpcMessageTag.SendChat:
                await this._handleSendChat(rpc);
                break;
            case constant_1.RpcMessageTag.StartMeeting:
                await this._handleStartMeeting(rpc);
                break;
            case constant_1.RpcMessageTag.SetPet:
                await this._handleSetPet(rpc);
                break;
            case constant_1.RpcMessageTag.SetStartCounter:
                await this._handleSetStartCounter(rpc);
                break;
            case constant_1.RpcMessageTag.UsePlatform:
                await this._handleUsePlatform(rpc);
                break;
            case constant_1.RpcMessageTag.SendQuickChat:
                await this._handleSendQuickChat(rpc);
                break;
        }
    }
    async _handleCompleteTask(rpc) {
        var _a;
        if (!((_a = this.player.info) === null || _a === void 0 ? void 0 : _a.taskStates))
            return;
        this._completeTask(rpc.taskidx);
        await this.emit(new events_1.PlayerCompleteTaskEvent(this.room, this.player, rpc, this.player.info.taskStates[rpc.taskidx]));
    }
    _completeTask(taskIdx) {
        var _a;
        (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.completeTask(this.playerId, taskIdx);
    }
    async _rpcCompleteTask(taskIdx) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.CompleteTaskMessage(taskIdx)));
    }
    /**
     * Mark one of this player's tasks as completed.
     * @param taskIdx The index of the task to complete. Note: this is not the
     * task ID but instead the index of the task in the {@link PlayerInfo.taskIds}
     * array.
     *
     * Emits a {@link PlayerCompleteTaskEvent | `player.completetask`} event.
     *
     * @example
     * ```ts
     * client.me.control.completeTask(0);
     * ```
     */
    completeTask(taskIdx) {
        var _a;
        if (!((_a = this.player.info) === null || _a === void 0 ? void 0 : _a.taskStates[taskIdx]))
            return;
        this.emit(new events_1.PlayerCompleteTaskEvent(this.room, this.player, undefined, this.player.info.taskStates[taskIdx]));
        this._completeTask(taskIdx);
        this._rpcCompleteTask(taskIdx);
    }
    async _handleSyncSettings(rpc) {
        if (!rpc.settings)
            return;
        this._syncSettings(rpc.settings);
        const ev = await this.emit(new events_1.PlayerSyncSettingsEvent(this.room, this.player, rpc, rpc.settings));
        if (ev.isDirty) {
            this._syncSettings(ev.alteredSettings);
            this._rpcSyncSettings(ev.alteredSettings);
        }
    }
    _syncSettings(settings) {
        this.room.settings.patch(settings);
    }
    async _rpcSyncSettings(settings) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SyncSettingsMessage(settings)));
    }
    /**
     * Change the room's settings. Use {@link Hostable.setSettings} to pass a
     * partial game objects object. This is a host operation on official servers.
     *
     * Emits a {@link PlayerSyncSettingsEvent | `player.syncsettings`} event.
     *
     * @property settings The full game settings object to update the settings to.
     */
    async syncSettings(settings) {
        this._syncSettings(settings);
        const ev = await this.emit(new events_1.PlayerSyncSettingsEvent(this.room, this.player, undefined, settings));
        this._syncSettings(ev.alteredSettings);
        this._rpcSyncSettings(settings);
    }
    async _handleSetImpostors(rpc) {
        const impostors = rpc.impostors
            .map((id) => this.room.getPlayerByPlayerId(id))
            .filter((player) => player && player.info);
        this._setImpostors(impostors);
        const ev = await this.emit(new events_1.PlayerSetImpostorsEvent(this.room, this.player, rpc, impostors));
        if (ev.isDirty) {
            this._setImpostors(ev.alteredImpostors);
            this._rpcSetImpostors(ev.alteredImpostors);
        }
    }
    _setImpostors(impostors) {
        for (const [, player] of this.room.players) {
            if (!player.info)
                continue;
            if (impostors.includes(player)) {
                player.info.setImpostor(true);
            }
            else {
                if (player.info.isImpostor) {
                    player.info.setImpostor(false);
                }
            }
        }
    }
    _rpcSetImpostors(impostors) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetInfectedMessage(impostors.map(impostor => impostor.playerId))));
    }
    /**
     * Set the impostors of the room. Usually called straight after a game is
     * started via the {@link InnerShipStatus.selectImpostors} method. This is
     * a host operation on official servers.
     *
     * Will remove any existing impostors unless they are included in the array.
     *
     * Emits a {@link PlayerSetImpostorsEvent | `player.setimpostors`} event.
     *
     * @param impostors An array of impostors to set.
     * @example
     * ```ts
     * // Set everyone with "Judas" in their name as the impostor.
     * awiat client.me.control.setImpostors(
     *   [...this.room.players.values()]
     *     .filter(player => player.info.name.includes("Judas"))
     * );
     * ```
     */
    async setImpostors(impostors) {
        this._setImpostors(impostors);
        const ev = await this.emit(new events_1.PlayerSetImpostorsEvent(this.room, this.player, undefined, impostors));
        if (ev.isDirty)
            this._setImpostors(ev.alteredImpostors);
        const clonedSettings = new protocol_1.GameSettings(this.room.settings);
        clonedSettings.numImpostors = impostors.length;
        this._rpcSyncSettings(clonedSettings);
        this._rpcSetImpostors(ev.alteredImpostors);
    }
    async _handleCheckName(rpc) {
        if (!this.room.gameData)
            return;
        let newName = rpc.name;
        const players = [...this.room.gameData.players.values()];
        if (players.some((player) => player.playerId !== this.playerId &&
            player.name.toLowerCase() === newName.toLowerCase())) {
            for (let i = 1; i < 100; i++) {
                newName = rpc.name + " " + i;
                if (!players.some((player) => player.playerId !== this.playerId &&
                    player.name.toLowerCase() === newName.toLowerCase())) {
                    break;
                }
            }
        }
        const ev = await this.emit(new events_1.PlayerCheckNameEvent(this.room, this.player, rpc, rpc.name, newName));
        if (!ev.canceled)
            await this.setName(ev.alteredName);
    }
    async _rpcCheckName(name) {
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.CheckNameMessage(name)),
        ], true, this.room.hostId);
    }
    /**
     * Request for the host to check a name for this player and append numbers if
     * it's taken.
     * @param name The name to request.
     */
    async checkName(name) {
        await this._rpcCheckName(name);
    }
    async _handleSetName(rpc) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldName = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.name;
        if (playerInfo)
            playerInfo.name = rpc.name;
        const ev = await this.emit(new events_1.PlayerSetNameEvent(this.room, this.player, rpc, oldName || "", rpc.name));
        if (playerInfo)
            playerInfo.name = ev.alteredName;
        if (ev.alteredName !== rpc.name)
            this._rpcSetName(ev.alteredName);
    }
    _rpcSetName(name) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetNameMessage(name)));
    }
    /**
     * Update this player's name. This is a host operation on official servers.
     * Use {@link PlayerControl.checkName} if you are calling this as not the host.
     *
     * Emits a {@link PlayerSetNameEvent | `player.setname`} event.
     *
     * @param name The name to set this player's name to.
     */
    async setName(name) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldName = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.name;
        if (playerInfo)
            playerInfo.name = name;
        const ev = await this.emit(new events_1.PlayerSetNameEvent(this.room, this.player, undefined, oldName || "", name));
        if (playerInfo)
            playerInfo.name = ev.alteredName;
        if (ev.alteredName !== oldName)
            this._rpcSetName(ev.alteredName);
    }
    async _handleCheckColor(rpc) {
        if (!this.room.gameData)
            return;
        let newColor = rpc.color;
        const players = [...this.room.gameData.players.values()];
        let i = 0;
        while (players.some((player) => player.playerId !== this.playerId &&
            player.color === newColor)) {
            newColor++;
            if (newColor >= 18)
                newColor = 0;
            i++;
            if (i >= 18)
                break;
        }
        const ev = await this.emit(new events_1.PlayerCheckColorEvent(this.room, this.player, rpc, rpc.color, newColor));
        if (!ev.canceled)
            this.setColor(ev.alteredColor);
    }
    async _rpcCheckColor(color) {
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.CheckColorMessage(color)),
        ], true, this.room.hostId);
    }
    /**
     * Request for the host to check a color for this player and change it if it's
     * taken.
     *
     * @param color The color to request.
     */
    async checkColor(color) {
        await this._rpcCheckColor(color);
    }
    async _handleSetColor(rpc) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldColor = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.color;
        if (playerInfo)
            playerInfo.color = rpc.color;
        const ev = await this.emit(new events_1.PlayerSetColorEvent(this.room, this.player, rpc, oldColor || constant_1.Color.Red, rpc.color));
        if (playerInfo)
            playerInfo.color = ev.alteredColor;
        if (ev.alteredColor !== rpc.color)
            this._rpcSetColor(ev.alteredColor);
    }
    _rpcSetColor(color) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetColorMessage(color)));
    }
    /**
     * Update this player's color. This is a host operation on official servers.
     * Use {@link PlayerControl.checkColor} if you are calling this as not the host.
     *
     * Emits a {@link PlayerCheckNameEvent | `player.checkname`} event.
     *
     * @param color The color to set this player's name to.
     */
    async setColor(color) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldColor = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.color;
        if (playerInfo)
            playerInfo.color = color;
        const ev = await this.emit(new events_1.PlayerSetColorEvent(this.room, this.player, undefined, oldColor || constant_1.Color.Red, color));
        if (playerInfo)
            playerInfo.color = ev.alteredColor;
        if (ev.alteredColor !== oldColor) {
            this._rpcSetColor(ev.alteredColor);
        }
    }
    async _handleSetHat(rpc) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldHat = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.hat;
        if (playerInfo)
            playerInfo.hat = rpc.hat;
        const ev = await this.emit(new events_1.PlayerSetHatEvent(this.room, this.player, rpc, oldHat || constant_1.Hat.None, rpc.hat));
        if (playerInfo)
            playerInfo.hat = ev.alteredHat;
        if (ev.alteredHat !== rpc.hat)
            this._rpcSetHat(ev.alteredHat);
    }
    _rpcSetHat(hat) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetHatMessage(hat)));
    }
    /**
     * Update this player's hat. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetHatEvent | `player.sethat`} event.
     *
     * @param hat The hat to set this player's hat to.
     */
    async setHat(hat) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldHat = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.hat;
        if (playerInfo)
            playerInfo.hat = hat;
        const ev = await this.emit(new events_1.PlayerSetHatEvent(this.room, this.player, undefined, oldHat || constant_1.Hat.None, hat));
        if (playerInfo)
            playerInfo.hat = ev.alteredHat;
        if (ev.alteredHat !== oldHat)
            this._rpcSetHat(ev.alteredHat);
    }
    async _handleSetSkin(rpc) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldSkin = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.skin;
        if (playerInfo)
            playerInfo.skin = rpc.skin;
        const ev = await this.emit(new events_1.PlayerSetSkinEvent(this.room, this.player, rpc, oldSkin || constant_1.Skin.None, rpc.skin));
        if (playerInfo)
            playerInfo.skin = ev.alteredSkin;
        if (ev.alteredSkin !== rpc.skin)
            this._rpcSetSkin(ev.alteredSkin);
    }
    _rpcSetSkin(skin) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetSkinMessage(skin)));
    }
    /**
     * Update this player's skin. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetSkinEvent | `player.setskin`} event.
     *
     * @param skin The skin to set this player's skin to.
     */
    async setSkin(skin) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldSkin = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.skin;
        if (playerInfo)
            playerInfo.skin = skin;
        const ev = await this.emit(new events_1.PlayerSetSkinEvent(this.room, this.player, undefined, oldSkin || constant_1.Skin.None, skin));
        if (playerInfo)
            playerInfo.skin = ev.alteredSkin;
        if (ev.alteredSkin !== oldSkin)
            this._rpcSetSkin(ev.alteredSkin);
    }
    async _handleReportDeadBody(rpc) {
        var _a, _b;
        const reportedBody = rpc.bodyid === 0xff
            ? "emergency"
            : this.room.getPlayerByPlayerId(rpc.bodyid);
        if (!reportedBody)
            return;
        const ev = await this.emit(new events_1.PlayerReportDeadBodyEvent(this.room, this.player, rpc, reportedBody));
        if (!ev.canceled) {
            (_b = (_a = this.room.host) === null || _a === void 0 ? void 0 : _a.control) === null || _b === void 0 ? void 0 : _b.startMeeting(this.player, ev.alteredBody);
        }
    }
    async _rpcReportDeadBody(body) {
        if (body !== "emergency" && body.playerId === undefined) {
            return this._rpcReportDeadBody("emergency");
        }
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.ReportDeadBodyMessage(body === "emergency"
                ? 0xff
                : body.playerId))
        ], true, this.room.hostId);
    }
    /**
     * Report the dead body of a player, telling the host to begin a meeting.
     * Use {@link PlayerControl.startMeeting} if you are calling this as the
     * host.
     *
     * Emits a {@link PlayerReportDeadBodyEvent `player.reportbody`} event.
     */
    async reportDeadBody(body) {
        const ev = await this.emit(new events_1.PlayerReportDeadBodyEvent(this.room, this.player, undefined, body));
        if (!ev.canceled) {
            await this._rpcReportDeadBody(ev.alteredBody);
        }
    }
    async _handleMurderPlayer(rpc) {
        const victim = this.room.getPlayerByNetId(rpc.victimid);
        if (!victim || victim.playerId === undefined)
            return;
        this._murderPlayer(victim);
        await this.emit(new events_1.PlayerMurderEvent(this.room, this.player, rpc, victim));
    }
    _murderPlayer(victim) {
        var _a;
        (_a = victim.info) === null || _a === void 0 ? void 0 : _a.setDead(true);
        if (!this.room.gameData)
            return;
        let aliveCrewmates = 0;
        let aliveImpostors = 0;
        for (const [, playerInfo] of this.room.gameData.players) {
            if (!playerInfo.isDisconnected && !playerInfo.isDead) {
                if (playerInfo.isImpostor) {
                    aliveImpostors++;
                }
                else {
                    aliveCrewmates++;
                }
            }
        }
        if (aliveCrewmates <= aliveImpostors) {
            this.room.registerEndGameIntent(new endgame_1.EndGameIntent(endgame_1.AmongUsEndGames.PlayersKill, constant_1.GameOverReason.ImpostorByKill, {
                killer: this.player,
                victim,
                aliveCrewmates,
                aliveImpostors
            }));
        }
    }
    _rpcMurderPlayer(victim) {
        if (!victim.control)
            return;
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.MurderPlayerMessage(victim.control.netId)));
    }
    /**
     * Murder another a player. This operation can only be called if the player
     * is the impostor on official servers.
     *
     * Due to technical limitations, this operation cannot be canceled or reverted
     * without advanced "breaking game", therefore it is out of scope of a single
     * `.revert()` function on the event emitted.
     *
     * Emits a {@link PlayerMurderEvent | `player.murder`} event.
     *
     * @param victim The player to murder.
     * @returns
     */
    murderPlayer(victim) {
        if (victim.playerId === undefined)
            return;
        this._murderPlayer(victim);
        this.emit(new events_1.PlayerMurderEvent(this.room, this.player, undefined, victim));
        this._rpcMurderPlayer(victim);
    }
    async _handleSendChat(rpc) {
        await this.emit(new events_1.PlayerSendChatEvent(this.room, this.player, rpc, rpc.message));
    }
    _rpcSendChat(message) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SendChatMessage(message)));
    }
    /**
     * Send a chat message as this player.
     *
     * Due to technical impossibilities, this event cannot be canceled or reverted.
     *
     * Emits a {@link PlayerSendChatEvent | `player.chat`} event.
     */
    sendChat(message) {
        this.emit(new events_1.PlayerSendChatEvent(this.room, this.player, undefined, message));
        this._rpcSendChat(message);
    }
    _rpcSendChatNote(player, type) {
        if (player.playerId === undefined)
            return;
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SendChatNoteMessage(player.playerId, type)));
    }
    /**
     * Send a player chat note for this player, i.e. "player x voted, x remaining.".
     */
    sendChatNote(player, type) {
        this._rpcSendChatNote(player, type);
    }
    async _handleStartMeeting(rpc) {
        const reportedBody = rpc.bodyid === 0xff
            ? "emergency"
            : this.room.getPlayerByPlayerId(rpc.bodyid) || "emergency";
        await this.emit(new events_1.PlayerStartMeetingEvent(this.room, this.player, rpc, reportedBody));
        if (this.room.hostIsMe) {
            this._startMeeting(this.player);
        }
    }
    _startMeeting(caller) {
        var _a;
        if (caller.playerId === undefined)
            return;
        const spawnMeetinghud = this.room.spawnPrefab(constant_1.SpawnType.MeetingHud, -2);
        const callerState = spawnMeetinghud.voteStates.get(caller.playerId);
        if (callerState) {
            callerState.didReport = true;
        }
        const movingPlatform = (_a = this.room.shipStatus) === null || _a === void 0 ? void 0 : _a.systems.get(constant_1.SystemType.GapRoom);
        if (movingPlatform instanceof systems_1.MovingPlatformSystem) {
            movingPlatform.setSide(systems_1.MovingPlatformSide.Left);
        }
    }
    _rpcStartMeeting(player) {
        if (player !== "emergency" && player.playerId === undefined) {
            return this._rpcStartMeeting("emergency");
        }
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.StartMeetingMessage(player === "emergency"
            ? 0xff
            : player.playerId)));
    }
    /**
     * Start a meeting and begin the meeting routine. This is a host-only operation
     * on official servers, see {@link PlayerControl.reportDeadBody} if you are calling
     * this as not the host.
     *
     * Emits a {@link PlayerStartMeetingEvent | `player.startmeeting`} event.
     *
     * @param caller The player that called this meeting.
     * @param body The body that was reported, or "emergency" if it is an emergency meeting.
     */
    startMeeting(caller, body) {
        this.emit(new events_1.PlayerStartMeetingEvent(this.room, this.player, undefined, body));
        this._rpcStartMeeting(body);
        this._startMeeting(caller);
    }
    async _handleSetPet(rpc) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        const oldPet = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.pet;
        if (playerInfo)
            playerInfo.pet = rpc.pet;
        const ev = await this.emit(new events_1.PlayerSetPetEvent(this.room, this.player, rpc, oldPet || constant_1.Pet.None, rpc.pet));
        if (playerInfo)
            playerInfo.pet = ev.alteredPet;
        if (ev.alteredPet !== rpc.pet)
            this._rpcSetPet(ev.alteredPet);
    }
    _rpcSetPet(pet) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetPetMessage(pet)));
    }
    /**
     * Update this player's pet. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetPetEvent | `player.setpet`} event.
     *
     * @param pet The pet to set this player's pet to.
     */
    async setPet(pet) {
        var _a;
        const playerInfo = await ((_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId));
        if (!playerInfo)
            return;
        const oldPet = playerInfo.pet;
        playerInfo.pet = pet;
        const ev = await this.emit(new events_1.PlayerSetPetEvent(this.room, this.player, undefined, oldPet, pet));
        playerInfo.pet = ev.alteredPet;
        if (playerInfo.pet !== oldPet) {
            this._rpcSetPet(playerInfo.pet);
        }
    }
    async _handleSetStartCounter(rpc) {
        const oldCounter = this.room.counter;
        this._setStartCounter(rpc.counter);
        const ev = await this.emit(new events_1.PlayerSetStartCounterEvent(this.room, this.player, rpc, oldCounter, rpc.counter));
        this.room.counter = ev.alteredCounter;
        if (this.room.counter !== rpc.counter) {
            await this._rpcSetStartCounter(ev.alteredCounter);
        }
    }
    _setStartCounter(counter) {
        this.lastStartCounter++;
        this.room.counter = counter;
    }
    _rpcSetStartCounter(counter) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetStartCounterMessage(this.lastStartCounter, counter)));
    }
    /**
     * Change the counter at the bottom of the screen while in the lobby,
     * usually counting down from 5 to 1. This is a host-only operation
     * on official servers.
     *
     * Emits a {@link PlayerSetStartCounterEvent | `player.setstartcounter`} event.
     *
     * @param counter The counter to set to.
     */
    async setStartCounter(counter) {
        this._setStartCounter(counter);
        const ev = await this.emit(new events_1.PlayerSetStartCounterEvent(this.room, this.player, undefined, this.room.counter, counter));
        if (ev.alteredCounter !== counter) {
            this.room.counter = ev.alteredCounter;
        }
        this._rpcSetStartCounter(ev.alteredCounter);
    }
    async _handleUsePlatform(rpc) {
        const airship = this.room.shipStatus;
        if (!airship || !(airship instanceof AirshipStatus_1.AirshipStatus))
            return;
        const ev = await this.emit(new events_1.PlayerUseMovingPlatformEvent(this.room, this.player, rpc));
        if (ev.canceled)
            return;
        this._usePlatform(false);
    }
    _usePlatform(rpc) {
        const airship = this.room.shipStatus;
        if (!airship || !(airship instanceof AirshipStatus_1.AirshipStatus))
            return;
        const movingPlatform = airship.systems.get(constant_1.SystemType.GapRoom);
        if (movingPlatform instanceof systems_1.MovingPlatformSystem) {
            movingPlatform.setTarget(this.player, movingPlatform.oppositeSide, rpc);
        }
    }
    /**
     * Use the moving platfrom on the map, i.e. the one on Airship.
     *
     * Emits a {@link MovingPlatformPlayerUpdateEvent | `movingplatform.playerupdate`} event.
     */
    async usePlatform() {
        const ev = await this.emit(new events_1.PlayerUseMovingPlatformEvent(this.room, this.player, undefined));
        if (ev.canceled)
            return;
        this._usePlatform(true);
    }
    async _handleSendQuickChat(rpc) {
        await this.emit(new events_1.PlayerSendQuickChatEvent(this.room, this.player, rpc, rpc.message));
    }
    _rpcSendQuickChat(message) {
        this.room.stream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SendQuickChatMessage(message)));
    }
    /**
     * Send a quick chat message as this player.
     *
     * Due to technical impossibilities, this event cannot be canceled or reverted.
     *
     * Emits a {@link PlayerSendChatEvent | `player.quickchat`} event.
     */
    sendQuickChat(message, format) {
        const quickChatMessage = typeof message === "number"
            ? format
                ? new protocol_1.QuickChatSentenceMessageData(message, format.map(format => {
                    return typeof format === "number"
                        ? format
                        : new protocol_1.QuickChatPlayerMessageData(format.playerId);
                }))
                : new protocol_1.QuickChatPhraseMessageData(message)
            : new protocol_1.QuickChatPlayerMessageData(message.playerId);
        this.emit(new events_1.PlayerSendQuickChatEvent(this.room, this.player, undefined, quickChatMessage));
        this._rpcSendQuickChat(quickChatMessage);
    }
}
exports.PlayerControl = PlayerControl;
//# sourceMappingURL=PlayerControl.js.map