"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HqHudSystem = exports.HqHudSystemRepairTag = void 0;
const SystemStatus_1 = require("./SystemStatus");
const events_1 = require("../events");
exports.HqHudSystemRepairTag = {
    CompleteConsole: 0x10,
    CloseConsole: 0x20,
    OpenConsole: 0x40
};
/**
 * Represents a system responsible for handling communication consoles on Mira HQ.
 *
 * See {@link HqHudSystemEvents} for events to listen to.
 */
class HqHudSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        var _a;
        super(ship, systemType, data);
        (_a = this.timer) !== null && _a !== void 0 ? _a : (this.timer = 10000);
        this.activeConsoles || (this.activeConsoles = []);
        this.completedConsoles || (this.completedConsoles = new Set([0, 1]));
    }
    get sabotaged() {
        return this.completedConsoles.size < 2;
    }
    _getIdx(consoleId, playerId) {
        return this.activeConsoles.findIndex((pair) => pair.consoleid === consoleId && pair.playerid === playerId);
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        const numActive = reader.upacked();
        const beforeActive = this.activeConsoles;
        this.activeConsoles = [];
        for (let i = 0; i < numActive; i++) {
            const playerId = reader.uint8();
            const consoleId = reader.uint8();
            const player = this.ship.room.getPlayerByPlayerId(playerId);
            if (player) {
                this._openConsole(consoleId, player, undefined);
            }
        }
        for (let i = 0; i < beforeActive.length; i++) {
            const console = beforeActive[i];
            const idx = this._getIdx(console.consoleid, console.playerid);
            const player = this.ship.room.getPlayerByPlayerId(console.playerid);
            if (player && idx === -1) {
                this._closeConsole(console.consoleid, player, undefined);
            }
        }
        const beforeCompleted = this.completedConsoles.size;
        const numCompleted = reader.upacked();
        for (let i = 0; i < numCompleted; i++) {
            this._completeConsole(reader.uint8(), undefined, undefined);
        }
        if (beforeCompleted === 2 && numCompleted === 0) {
            this.emit(new events_1.SystemSabotageEvent(this.room, this, undefined, undefined));
        }
        if (beforeCompleted < 2 && numCompleted === 2) {
            this.emit(new events_1.SystemRepairEvent(this.room, this, undefined, undefined));
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        writer.upacked(this.activeConsoles.length);
        for (let i = 0; i < this.activeConsoles.length; i++) {
            const active = this.activeConsoles[i];
            writer.uint8(active.playerid);
            writer.uint8(active.consoleid);
        }
        const completed = [...this.completedConsoles];
        writer.upacked(completed.length);
        for (let i = 0; i < completed.length; i++) {
            writer.uint8(completed[i]);
        }
    }
    async HandleSabotage(player, rpc) {
        const oldTimer = this.timer;
        const oldActive = this.activeConsoles;
        const oldCompleted = this.completedConsoles;
        this.timer = -1;
        this.activeConsoles = [];
        this.completedConsoles = new Set;
        this.dirty = true;
        const ev = await this.emit(new events_1.SystemSabotageEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.timer = oldTimer;
            this.activeConsoles = oldActive;
            this.completedConsoles = oldCompleted;
        }
    }
    async _resetConsoles() {
        const oldCompleted = this.completedConsoles;
        this.completedConsoles = new Set;
        this.dirty = true;
        const ev = await this.emit(new events_1.HqHudConsolesResetEvent(this.room, this));
        if (ev.reverted) {
            this.completedConsoles = oldCompleted;
        }
    }
    async _openConsole(consoleid, player, rpc) {
        if (player.playerId === undefined)
            return;
        const idx = this._getIdx(consoleid, player.playerId);
        if (idx === -1) {
            const consoleEntry = { consoleid, playerid: player.playerId };
            this.activeConsoles.push(consoleEntry);
            this.dirty = true;
            const ev = await this.emit(new events_1.HqHudConsoleOpenEvent(this.room, this, rpc, player, consoleid));
            if (ev.reverted) {
                const newIdx = this.activeConsoles.indexOf(consoleEntry);
                if (newIdx > -1)
                    this.activeConsoles.splice(newIdx, 1);
                return;
            }
            if (ev.alteredConsoleId !== consoleid) {
                consoleEntry.consoleid = ev.alteredConsoleId;
            }
        }
    }
    async openConsoleAs(consoleId, player) {
        await this._openConsole(consoleId, player, undefined);
    }
    /**
     * Mark the console as being used by your player.
     * @param consoleId The ID of the console to mark as being used.
     */
    async openConsole(consoleId) {
        if (!this.room.myPlayer)
            return;
        if (this.room.hostIsMe) {
            await this.openConsoleAs(consoleId, this.room.myPlayer);
        }
        else {
            this._sendRepair(0x40 | consoleId);
        }
    }
    async _closeConsole(consoleid, player, rpc) {
        if (player.playerId === undefined)
            return;
        const idx = this._getIdx(consoleid, player.playerId);
        if (idx > -1) {
            const consoleEntry = this.activeConsoles[idx];
            this.activeConsoles.splice(idx, 1);
            this.dirty = true;
            const ev = await this.emit(new events_1.HqHudConsoleCloseEvent(this.room, this, rpc, player, consoleid));
            if (ev.reverted) {
                this.activeConsoles.splice(idx, 0, consoleEntry);
            }
        }
    }
    async closeConsoleAs(consoleId, player) {
        await this._closeConsole(consoleId, player, undefined);
    }
    /**
     * Mark the console as no longer being used by your player.
     * @param consoleId The ID of the console to mark as not being used.
     */
    async closeConsole(consoleId) {
        if (!this.room.myPlayer)
            return;
        if (this.room.hostIsMe) {
            await this.closeConsoleAs(consoleId, this.room.myPlayer);
        }
        else {
            await this._sendRepair(0x20 | consoleId);
        }
    }
    async _completeConsole(consoleid, player, rpc) {
        if (!this.completedConsoles.has(consoleid)) {
            this.completedConsoles.add(consoleid);
            this.dirty = true;
            const ev = await this.emit(new events_1.HqHudConsoleCompleteEvent(this.room, this, rpc, player, consoleid));
            if (ev.reverted) {
                this.completedConsoles.delete(consoleid);
            }
        }
    }
    /**
     * Mark a console as completed.
     * @param consoleId The ID of the console to mark as completed.
     */
    async completeConsole(consoleId) {
        if (this.room.hostIsMe) {
            await this._completeConsole(consoleId, this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(0x10 | consoleId);
        }
    }
    async _repair(player, rpc) {
        const completedBefore = this.completedConsoles;
        const timerBefore = this.timer;
        this.completedConsoles = new Set([0, 1]);
        this.timer = -1;
        this.dirty = true;
        const ev = await this.emit(new events_1.SystemRepairEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.timer = timerBefore;
            this.completedConsoles = completedBefore;
        }
    }
    async repair() {
        if (this.room.hostIsMe) {
            await this._repair(this.room.myPlayer, undefined);
        }
        else {
            await this.completeConsole(0);
            await this.completeConsole(1);
        }
    }
    async HandleRepair(player, amount, rpc) {
        const consoleId = amount & 0xf;
        const repairOperation = amount & 0xf0;
        switch (repairOperation) {
            case exports.HqHudSystemRepairTag.CompleteConsole:
                await this._completeConsole(consoleId, player, rpc);
                if (this.completedConsoles.size >= 2) {
                    await this._repair(player, rpc);
                }
                break;
            case exports.HqHudSystemRepairTag.CloseConsole:
                if (!player)
                    return;
                await this._closeConsole(consoleId, player, rpc);
                break;
            case exports.HqHudSystemRepairTag.OpenConsole:
                if (!player)
                    return;
                await this._openConsole(consoleId, player, rpc);
                break;
        }
    }
    Detoriorate(delta) {
        if (!this.sabotaged)
            return;
        this.timer -= delta;
        if (this.timer < 0) {
            this.timer = 10;
            this.dirty = true;
            this._resetConsoles();
        }
    }
}
exports.HqHudSystem = HqHudSystem;
//# sourceMappingURL=HqHudSystem.js.map